@@ -75,24 +75,24 @@ sub from_datetimes {
     return $self;
 }
 
-*new = \&from_datetimes;
-
 sub from_datetime_and_duration {
-    my $self = shift;
+    my $class = shift;
     my %args = @_;
-    my %date;
+
+    my $key;
+    my $dt;
     # extract datetime parameters
     for ( qw( start end before after ) ) {
-        $date{$_} = delete $args{$_} if exists $args{$_};
+        if ( exists $args{$_} ) {
+           $key = $_;
+           $dt = delete $args{$_};
+       }
     }
-    my ($key) = keys %date;
-    my $dt = $date{$key};
+
     # extract duration parameters
-    my ($duration_key) = keys %args;
     my $dt_duration;
-    # warn "args: $key - $duration_key - @{[ %args ]}";
-    if ($duration_key eq 'duration') {
-        $dt_duration = $args{$duration_key};
+    if ( exists $args{duration} ) {
+        $dt_duration = $args{duration};
     }
     else {
         $dt_duration = DateTime::Duration->new( %args );
@@ -113,7 +113,26 @@ sub from_datetime_and_duration {
         $other_key = 'before' if $key eq 'start';
         $key = 'start';
     }
-    return $self->new( $key => $dt, $other_key => $other_date ); 
+    return $class->new( $key => $dt, $other_key => $other_date ); 
+}
+
+# This method is intentionally not documented.  It's really only for
+# use by ::Set and ::SpanSet's as_list() and iterator() methods.
+sub new {
+    my $class = shift;
+    my %args = @_;
+
+    # If we find anything _not_ appropriate for from_datetimes, we
+    # assume it must be for durations, and call this constructor.
+    # This way, we don't need to hardcode the DateTime::Duration
+    # parameters.
+    foreach ( keys %args )
+    {
+        return $class->from_datetime_and_duration(%args)
+            unless /^(?:before|after|start|end)$/;
+    }
+
+    return $class->from_datetimes(%args);
 }
 
 sub clone { 
@@ -127,12 +146,12 @@ sub clone {
 sub intersection {
     my ($set1, $set2) = @_;
     my $class = ref($set1);
-    my $tmp = $class->new();
+    my $tmp = {};  # $class->new();
     $set2 = DateTime::Set->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
     $tmp->{set} = $set1->{set}->intersection( $set2->{set} );
 
     # intersection() can generate something more complex than a span.
-    bless $tmp, 'Date::SpanSet';
+    bless $tmp, 'DateTime::SpanSet';
 
     return $tmp;
 }
@@ -156,12 +175,12 @@ sub contains {
 sub union {
     my ($set1, $set2) = @_;
     my $class = ref($set1);
-    my $tmp = $class->new();
+    my $tmp = {};   # $class->new();
     $set2 = DateTime::Set->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
     $tmp->{set} = $set1->{set}->union( $set2->{set} );
  
     # union() can generate something more complex than a span.
-    bless $tmp, 'Date::SpanSet';
+    bless $tmp, 'DateTime::SpanSet';
 
     # # We have to check it's internal structure to find out.
     # if ( $#{ $tmp->{set}->{list} } != 0 ) {
@@ -174,7 +193,7 @@ sub union {
 sub complement {
     my ($set1, $set2) = @_;
     my $class = ref($set1);
-    my $tmp = $class->new();
+    my $tmp = {};   # $class->new;
     if (defined $set2) {
         $set2 = DateTime::Set->new( dates => [ $set2 ] ) unless $set2->can( 'union' );
         $tmp->{set} = $set1->{set}->complement( $set2->{set} );
@@ -184,7 +203,7 @@ sub complement {
     }
 
     # complement() can generate something more complex than a span.
-    bless $tmp, 'Date::SpanSet';
+    bless $tmp, 'DateTime::SpanSet';
 
     # # We have to check it's internal structure to find out.
     # if ( $#{ $tmp->{set}->{list} } != 0 ) {
@@ -228,7 +247,7 @@ sub end_is_closed { $_[0]->end_is_open ? 0 : 1 }
 # span == $self
 sub span { @_ }
 
-sub duration { my $dur = $_[0]->{set}->size; defined $dur ? $dur : INFINITY }
+sub duration { my $dur = $_[0]->end - $_[0]->start; defined $dur ? $dur : INFINITY }
 *size = \&duration;
 
 # unsupported Set::Infinite methods
@@ -280,36 +299,36 @@ Creates a new span based on a starting and ending datetime.
 
 A 'closed' span includes its end-dates:
 
-   $dates = DateTime::Set->new( start => $dt1, end => $dt2 );
+   $span = DateTime::Span->from_datetimes( start => $dt1, end => $dt2 );
 
 An 'open' span does not include its end-dates:
 
-   $dates = DateTime::Set->new( after => $dt1, before => $dt2 );
+   $span = DateTime::Span->from_datetimes( after => $dt1, before => $dt2 );
 
 A 'semi-open' span includes one of its end-dates:
 
-   $dates = DateTime::Set->new( start => $dt1, before => $dt2 );
-   $dates = DateTime::Set->new( after => $dt1, end => $dt2 );
+   $span = DateTime::Span->from_datetimes( start => $dt1, before => $dt2 );
+   $span = DateTime::Span->from_datetimes( after => $dt1, end => $dt2 );
 
 A span might have just a beginning date, or just an ending date.
 These spans end, or start, in an imaginary 'forever' date:
 
-   $dates = DateTime::Set->new( start => $dt1 );
-   $dates = DateTime::Set->new( end => $dt2 );
-   $dates = DateTime::Set->new( after => $dt1 );
-   $dates = DateTime::Set->new( before => $dt2 );
+   $span = DateTime::Span->from_datetimes( start => $dt1 );
+   $span = DateTime::Span->from_datetimes( end => $dt2 );
+   $span = DateTime::Span->from_datetimes( after => $dt1 );
+   $span = DateTime::Span->from_datetimes( before => $dt2 );
 
 You cannot give both a "start" and "after" argument, nor can you give
 both an "end" and "before" argument.  Either of these conditions cause
-will cause the C<new()> method to die.
+will cause the C<from_datetimes()> method to die.
 
 =item * from_datetime_and_duration
 
 Creates a new span.
 
-   $dates = DateTime::Set->from_datetime_and_duration( 
+   $span = DateTime::Span->from_datetime_and_duration( 
        start => $dt1, duration => $dt_dur1 );
-   $dates = DateTime::Set->from_datetime_and_duration( 
+   $span = DateTime::Span->from_datetime_and_duration( 
        after => $dt1, hours => 12 );
 
 The new "end of the set" is I<open> by default.
@@ -342,17 +361,20 @@ objects, but also with C<DateTime::Set> and C<DateTime::SpanSet>
 objects.  These set operations always return a C<DateTime::SpanSet>
 object.
 
-    $set = $set1->union( $set2 );         # like "OR", "insert", "both"
-    $set = $set1->complement( $set2 );    # like "delete", "remove"
-    $set = $set1->intersection( $set2 );  # like "AND", "while"
-    $set = $set1->complement;             # like "NOT", "negate", "invert"
+    $set = $span->union( $set2 );         # like "OR", "insert", "both"
+    $set = $span->complement( $set2 );    # like "delete", "remove"
+    $set = $span->intersection( $set2 );  # like "AND", "while"
+    $set = $span->complement;             # like "NOT", "negate", "invert"
 
 =item intersects / contains
 
 These set functions return a boolean value.
 
-    if ( $set1->intersects( $set2 ) ) { ...  # like "touches", "interferes"
-    if ( $set1->contains( $set2 ) ) { ...    # like "is-fully-inside"
+    if ( $span->intersects( $set2 ) ) { ...  # like "touches", "interferes"
+    if ( $span->contains( $dt ) ) { ...    # like "is-fully-inside"
+
+These methods can accept a C<DateTime>, C<DateTime::Set>,
+C<DateTime::Span>, or C<DateTime::SpanSet> object as an argument.
 
 =back
 
